% AIMS:  Project the COARSE and FINE growth map back to the 3D Bone Geometry
% 
% STEPS: 1. Try.... start with UVF... 

% COMMENT: 1. The previous code is 'AnalysisFin2D.m'
%          2. 'AnalysisFin2D.m' --> iStep = 1 generates the 'wildM1UVF.m' 
%              iStep = 2 plot the results of decomposition and denoise of
%              UVF map. 
%          3. 'wildM1UVF.mat' --> the unwraped growth map (i.e.,theta-Z MAP)
%          4. 'wildM1F_wk02.mat' --> Geometry change F (cMap) and the
%              isosurface of the geometry. 
% %          5. UVF coordinates, 
%             UVF(:,1) - U is angle# to median point
%             UVF(:,2) - V is the bone height == z axis == Idx(:,3)
%             UVF(:,3) - F is the growth rate -F*10.4. ==> cMap, growth rate (microns)
%          6. UVFinfo(iS, iW - 1) => 'Mice': ['wildM', num2str(iMice)]
%                                    'Weeks': Weeks, 
%                                    'faceName': faceName{iS}, 
%                                    'UVF': UVF 
%                                    'XZmap': X, 
%                                    'zlimits': Z1 
%                                    'xlimits':[-179:180]; 
% Time: Nov. 2017
% ----------------------------------------------------------------------- %
clear all; 
% close all; 

% ------- adding pathes and setup RdBu colormap
addpath functions
addpath wild_Cropped
addpath('functions/wvDenoise');
addpath('functions/cbrewer');
RdYlBu= flipud(cbrewer('div', 'RdYlBu', 256)); 
RdBu= flipud(cbrewer('div', 'RdBu', 256));
signPlot = 1% 1--> Plot  0--> no Plots 

close all;
tic
for iMice = 5%2:5
    fileName_UVFL = ['wildM', num2str(iMice), 'UVFL']; 
    load(fileName_UVFL); % download unWraped map  

    for iW = 2
        Weeks = [0, iW];
        loadFileName = ['wildM', num2str(iMice),'F_wk0', num2str(iW)];
        load(loadFileName); % Load original bone surfaces     
        for iS = 1:2
            display([' Processing week: ', num2str(iW), ' Surface: ', num2str(iS)]);
            thisBone = []; thisBone = UVFinfo(iS, iW-1);
            X = []; X = thisBone.XZmap;
            
        % ------ DEFINE the boundary condition
            padSize = (1792 - size(X, 1))/2; %1856 1920
            X0 = []; X0 = padarray(X, padSize); %152
            cirSize = 76;
            X0 = padarray(X0, [0,cirSize], 'circular');
            X0(isnan(X0)) = 0;
            
        % ----- Dual tree complex wavelet analysis of signal X0           
            Depth = 4; % levels
            decompF =  dddtree2('cplxdt',X0,Depth,'FSfarras','qshift14');
            cfs = decompF.cfs;   % COEFFICIENTS       

        % ----- Denoising  
            % % --- Soft Thresholding 
            for j = 1:Depth
                for s1 = 1:2 % Go throught the tree
                    for s2 = 1:3 % Direction
                        C1 = []; C1 = cfs{j}(:,:, s2, s1,1) ;
                        C2 = []; C2 = cfs{j}(:,:, s2, s1,2);
                        C1 (isnan(C1)) = 0;
                        C2 (isnan(C2)) = 0;
                        HH = [];HH = abs(cfs{j}(:,:, s2, s1, 1) + i * cfs{j}(:,:, s2, s1, 2));
                        HH (isnan(HH)) = 0;
                        Nsig = median(abs(HH(:)))/0.6745 ;
                        C =  []; C = wthresh(C1 + i*C2, 's',Nsig);
                       cfs{j}(:,:, s2, s1, 1) = real(C);
                       cfs{j}(:,:, s2, s1, 2) = imag(C);
                    end
                end
            end

       % -----  Recovery
            % % --- 1 Coarse Scale
            dtcplx_coarse = decompF;  % decompF -- > Before denoise
            dtcplx_coarse.cfs = cfs; 
            for iD = 1:Depth
                dtcplx_coarse.cfs{iD} = zeros(size(dtcplx_coarse.cfs{iD}));
            end
            X_coarse = idddtree2(dtcplx_coarse); %Inverse dual-tree and double-density 2-D wavelet transform
            X_coarse(1:padSize-1, :) = [];    X_coarse(end-padSize: end, :) = [];
            X_coarse(:,1:cirSize-1) = [];     X_coarse(:, end- cirSize:end) = []; 
         
            % % --- 2 Fine Scale
            dtcplx_fine = decompF;  
            dtcplx_fine.cfs = cfs;
            dtcplx_fine.cfs{Depth + 1} = zeros(size(dtcplx_fine.cfs{Depth + 1}));
            X_fine = idddtree2(dtcplx_fine); % Inverse dual-tree and double-density 2-D wavelet transform
            X_fine(1:padSize-1, :) = [];    X_fine(end-padSize: end, :) = [];
            X_fine(:,1: cirSize-1) = [];    X_fine(:, end-cirSize:end) = [];

            U = []; U = round(thisBone.UVF(:,1)); U(U == -180) = 180;
            U = U - thisBone.xlimits(1) + 1;
            V = []; V = round(thisBone.UVF(:,2));
            V = V - thisBone.zlimits(1) + 1; 
            try
                F_coarse = []; F_coarse = X_coarse(sub2ind(size(X_coarse),V, U));
            catch
%                 U(V == max(V) ) = [];
%                 V(V == max(V) ) = [];
                X_coarse = [X_coarse(1,:); X_coarse]; 
                X_fine = [X_fine(1,:); X_fine]; 
                X = [X(1,:); X];
                F_coarse = []; F_coarse = X_coarse(sub2ind(size(X_coarse),V, U));
              
            end
            F_fine = []; F_fine = X_fine(sub2ind(size(X_fine),V, U));
            F = []; F = X(sub2ind(size(X),V, U));
           
            switch iS
                case 1
                    thisIso = []; thisIso = s_per(1); % vertices + faces
                        recovF(iMice, iW).perF = F       ; 
                case 2
                    thisIso = []; thisIso = s_end(1); 
                        recovF(iMice, iW).endF = F        ;
            end
            
            if signPlot
                % -- Got my cylindar
                Rho = ones(length(thisBone.UVF(:, 1)),1)*100;
                theta = thisBone.UVF(:,1);
                z = thisBone.UVF(:,2);
                [cx, cy, cz] = pol2cart(theta, Rho, z);
                RdBu= flipud(cbrewer('div', 'RdBu', 300));
                idx = []; idx = round(thisBone.UVF(:,3))+ 150; 
                idx(idx > 300) = 300; idx(idx < 1) = 1;
                cmap = RdBu(idx , :);
                colormap(RdBu);
                bonePlot(thisIso, thisBone.UVF(:,3));                                
                axis off  
                hold on; 
                caxis([-150,150])
               
    % == coarse                                       
            title({['Original'],['WildM', num2str(iMice), 'W', num2str(iW), 'S', num2str(iS)]}, ...
            'fontSize', 20);
            % print(['wildPTH_Cropped/OrginalWildPTHM', num2str(iMice), 'W', num2str(iW), 'S', num2str(iS)], '-r300', '-dpng');
            bonePlot(thisIso, F_coarse);
            caxis([-150, 150])
            title(['week 14 - ', num2str(iW+14)], 'fontSize', 14);
            axis off
    % print(['wildPTH_Cropped/RecoveredCoarseWildPTHM', num2str(iMice), 'W', num2str(iW), 'S', num2str(iS)], '-r300', '-dpng');
     
    % == fine
             bonePlot(thisIso, F_fine);
              hold on; caxis([-40,40])
              title(['week 14 - ', num2str(iW+14)], 'fontSize', 14);
              axis off
               
            end
        end
    end
end


