%% yzhLS_CompareWeeksLSF.m   
%  Statistical analysis of the growth speed F at normal direction
%  Input: TIFF image sequences
%  Output: F = LSF(t+1) - LSF(t) and represent F by colors

clear all;
close all;
addpath functions;
addpath('functions/cbrewer')
% voxel size in mm  voxel = 10.4; 

%% =================================================================== %
%  1. Reading data
%  2. Construct the level set function 
%  3. All the weeks are compared to week 0. [0,2] [0,3]...[0,8]
tic
% processedWeeks = [2:8];
processedWeeks = 2;
for ww = processedWeeks
    ww
    clearvars -except ww;
    Weeks = [0, ww]; 
    % Input 
    iMice = 5;  % Which mouse to process
    filePath = '..\WildM5_Mask\'
    
    h = waitbar(0, 'I am running...');
    waitbar(1/10);
for wk = 1:length(Weeks)   
    idx = []; idx = find(filePath == '\');
    filePath1 = [filePath(1:idx(end-1)), 'WildM',num2str(iMice), '_Mask\',...
        'WildM',num2str(iMice),'W',num2str(Weeks(wk)), '_Mask']; % -- Change the folder acoording to the experience...
    addpath(filePath1);
    d = []; d = dir(fullfile(filePath1));
    dirOutput=d(~ismember({d.name},{'.','..'}));
    fileNames = {dirOutput.name}';
    NFrames = numel(fileNames)
    Img = [];
    for iF = 1:NFrames
        crossView = []; 
        crossView = fileNames{iF};
        Y = [];
        Y = imread(crossView);
        Y= im2double(Y);
        Img(:,:,iF) = Y;
    end
    % Make sure all the lsf at the same size 640 * 460 * 1860
    % for wildM3 640*500
    if size(Img, 1) > 460
       Img(461:end, :,:) = [];
    end
%     Img(:,1:10,:) = [];
    if size(Img, 2) > 480
       Img(:,481:end,:) = [];
    end
    % Make sure all the lsf at the same size 
    padSize = [460, 480, 1770] - size(Img);
    Ibw = [];
    Ibw = padarray(Img, padSize, 'post');
    % construct the level set function    
    if isempty (find(Ibw>0 & Ibw < 0))  % If it is grayscale
         LSF0{wk} = yzhReinitilization(Ibw);
    else
        I = Ibw; I(I == 0) = [];
        thd = graythresh(I) %0.5412;
        clear I
        Ibw = Ibw - thd;
        LSF0{wk} = yzhReinitilization(Ibw);
    end
 
end

   clearvars -except LSF0 Weeks ww iMice h RdBu;
   waitbar(2/10);
%% ================================================================== %
w1 = 1; w2 = 2;
for wk =  [w1, w2]
    [lsf_per{wk}, lsf_end{wk},~, ~] = yzhSplitLSF(LSF0{wk},0);    
    s_per(wk) = isosurface(lsf_per{wk},0);
    s_end(wk) = isosurface(lsf_end{wk},0);
    waitbar((wk+2)/10)
end

F_per = lsf_per{w2} - lsf_per{w1};
F_end = lsf_end{w2} - lsf_end{w1};
waitbar(5/10)

% Periosteum
cMap_per = [];
Idx = []; Idx = round(s_per(1).vertices); Idx(Idx==0) = 1; idx_per = Idx;
idx = []; 
idx = (Idx(:, 3)-1) * size(F_per,2) * size(F_per,1) + (Idx(:,1) - 1) * size(F_per,1) + Idx(:,2) ; 
cMap_per = -1*F_per(idx) * 10.4 ; %--- periosteum 
waitbar(7/10)
    
cMap_end = []; %--> Endosteum
Idx = []; Idx = round(s_end(1).vertices); Idx(Idx==0) = 1;  idx_end = Idx;
idx = []; 
idx = (Idx(:, 3)-1) * size(F_end,2) * size(F_end,1) + (Idx(:,1) - 1) * size(F_end,1) + Idx(:,2) ;  
cMap_end = -1*F_end(idx) * 10.4 ;
waitbar(9/10)

% % % % % -- Define your saved name....
saveFname = ['wild_Cropped/wildM', num2str(iMice), 'F_wk', num2str(Weeks(1)),num2str(Weeks(2))];
save(saveFname, 's_per', 's_end','idx_per', 'idx_end', ...
         'cMap_per','cMap_end');
% % % % % -- Define your saved name....

waitbar(10/10)
close(h)

%% ======= Results plotting ============ % 
RdBu= flipud(cbrewer('div', 'RdBu', 256));
figure; clf;
hold on; grid on; set(gcf,'color',[1,1,1])    
set(gcf, 'Units', 'normalized', 'position', [0.2, 0.2, 0.24, 0.6])
patch(s_per(1),'FaceVertexCData',cMap_per,'FaceColor','flat','EdgeColor','none');
% patch(s_per(2),'FaceColor','g','EdgeColor','none');
axis equal;
view(3);
colormap(RdBu);
caxis([-150,150])
lighting gouraud;
camlight left; 
colorbar('Position',...
        [0.75 0.2 0.025 0.65]);
title({'Periosteum', ['wildM', num2str(iMice),'W',  num2str(Weeks(1)),...
    num2str(Weeks(2))]});
saveFig = []; 
saveFig = ['wild_Cropped/wildM', num2str(iMice), 'Fper3D_wk', num2str(Weeks(1)),...
    num2str(Weeks(2))];
% print(saveFig, '-dpng', '-r300') % --- save image

figure;clf;
hold on; grid on; set(gcf,'color',[1,1,1])   ;
set(gcf, 'Units', 'normalized', 'position', [0.2, 0.2, 0.24, 0.6])
patch(s_end(1),'FaceVertexCData',cMap_end,'FaceColor','flat','EdgeColor','none');
axis equal;
view(3);
colormap (RdBu)
caxis([-150,150])
lighting gouraud;
camlight left; 
colorbar('Position',[0.75 0.2 0.025 0.65]);
title({'Endosteum', ['wildPTHM', num2str(iMice),'W', num2str(Weeks(1)),...
    num2str(Weeks(2))]}); 
saveFig = []; 
saveFig = ['wild_Cropped/wildPTHM' num2str(iMice),'Fend3D_wk', num2str(Weeks(1)),...
    num2str(Weeks(2))];
% print(saveFig, '-dpng', '-r300')
end
